--
-- The MIT License (MIT)
--
-- Copyright (c) 2015 Kaadmy
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.
--

decals = {}

function decals.register_decal(dtype, desc, img, lifetime, chance)
   local chancel = chance
   minetest.register_node(
      "decals:decal_" .. dtype,
      {
	 description = desc,

	 drawtype = "nodebox",

	 node_box = {
	    type = "fixed",
	    fixed = {-0.5, -0.5, -0.5, 0.5, -0.5 + (1 / 64), 0.5},
	 },

	 tiles = {img, "decals_empty.png", "decals_empty.png"},
	 paramtype = "light",

	 walkable = false,
	 buildable_to = true,
	 pointable = false,
	 sunlight_propagates = true,
	 
	 groups = {not_in_creative_inventory = 1},

	 on_construct = function(pos)
			   if math.random() < chancel then
			      minetest.get_node_timer(pos):start(lifetime*((math.random()*0.75)+0.25))
			   else
			      minetest.remove_node(pos)
			   end
			end,
	 on_timer = function(pos, elapsed)
		       minetest.remove_node(pos)
		    end,
      })
end

decals.register_decal(
   "blood", -- name
   "Blood", -- description
   "decals_decal_blood.png", -- image
   15, -- maxiumum time the decal lasts in seconds
   0.7 -- chance of being spawned, from 0(0%) to 1(100%)
)

function decals.add_decal(pos, dtype, radius)
   local function decal(pos)
      -- add a single blood node on top of another node
      local under = minetest.get_node(pos).name
      if under == "air" then return end

      local under_def = minetest.registered_nodes[under]
      if under_def.drawtype ~= "normal" then return end
      
      local above = {x = pos.x, y = pos.y+1, z = pos.z}
      if minetest.get_node(above).name ~= "air" then return end

      minetest.set_node(above, {name = "decals:decal_" .. dtype})
   end

   for y = radius, -radius*4, -1 do -- top to bottom, go down farther than other axis
      for x = -radius, radius do
	 for z = -radius, radius do
	    local bp = {x = pos.x+x, y = pos.y+y, z = pos.z+z}
	    decal(bp)
	 end
      end
   end
end

minetest.register_craftitem(
   "decals:tool",
   {
      description = "Decal Tool",
      
      inventory_image = "decals_tool.png",

      on_use = function(itemstack, user, pointed_thing)
		  if pointed_thing.type == "node" then
		     decals.add_decal(pointed_thing.under, "blood", 1)
		  end
	       end,
   })